<?php
/**
 * Stygian Theme Customizer
 *
 * @package Stygian
 */

/**
 * Add postMessage support for site title and description for the Theme Customizer.
 *
 * @param WP_Customize_Manager $wp_customize Theme Customizer object.
 */
function stygian_sortable_sections( $wp_customize ) {
	$wp_customize->add_panel( 'stygian_sp_sortable', array(
		'title'       => esc_html__( 'Sections', 'stygian' ),
		'priority'    => 150,
	) );

	$default_sections = stygian_get_default_sortable_sections();

	$sortable_options = array();

	$sortable_order = stygian_gtm( 'stygian_ss_order' );

	if ( $sortable_order ) {
		$sortable_options = explode( ',', $sortable_order );
	}

	$sortable_sections = array_unique( $sortable_options + array_keys( $default_sections ) );
		
	foreach ( $sortable_sections as $section ) {
		if ( isset( $default_sections[$section] ) ) {
			// Add sections.
			$wp_customize->add_section( 'stygian_ss_' . $section,
				array(
					'title' => $default_sections[$section],
					'panel' => 'stygian_sp_sortable'
				)
			);
		}
		
		unset($default_sections[$section]);
	}

	if ( count( $default_sections ) ) {
		foreach ($default_sections as $key => $value) {
			// Add new sections.
			$wp_customize->add_section( 'stygian_ss_' . $key,
				array(
					'title' => $value,
					'panel' => 'stygian_sp_sortable'
				)
			);
		}
	}

	// Add hidden section for saving sorted sections.
	Stygian_Customizer_Utilities::register_option(
		array(
			'settings'          => 'stygian_ss_order',
			'sanitize_callback' => 'sanitize_text_field',
			'label'             => esc_html__( 'Section layout', 'stygian' ),
			'section'           => 'stygian_ss_main_content',
		)
	);
}
add_action( 'customize_register', 'stygian_sortable_sections', 1 );

/**
 * Default sortable sections order
 * @return array
 */
function stygian_get_default_sortable_sections() {
	return $default_sections = array (
		'slider'           => esc_html__( 'Slider', 'stygian' ),
		'wwd'              => esc_html__( 'What We Do', 'stygian' ),
		'hero_content'     => esc_html__( 'Hero Content', 'stygian' ),
		'featured_grid'    => esc_html__( 'Featured Grid', 'stygian' ),
		'testimonial'      => esc_html__( 'Testimonials', 'stygian' ),
		'contact_form'     => esc_html__( 'Contact Form', 'stygian' ),
	);
}
